# Generated by Django 5.0.9 on 2024-12-05 09:00

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("database", "0075_migrate_generated_assets_and_validate"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="OpenAIProcessorConversationConfig",
            new_name="AiModelApi",
        ),
        migrations.RenameField(
            model_name="chatmodeloptions",
            old_name="openai_config",
            new_name="ai_model_api",
        ),
        migrations.RenameField(
            model_name="texttoimagemodelconfig",
            old_name="openai_config",
            new_name="ai_model_api",
        ),
    ]
